/*
 Cmd.h - base class for Cmd objects

 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall.

 Modified by Will 18, 2006: will@opaline.demon.co.uk
 Added capacity to support an undo list
*/



#ifndef CMD_H
#define CMD_H

#include "List.h"
#include "CmdInterface.h"

class CmdList;


class Cmd
{
	friend class CmdInterface;
	friend class CmdList;
	
	public:
		virtual ~Cmd();
		enum CmdType { NOP, Action, Toggle, Separator, myList };

		virtual void execute();
		virtual void undo();
		virtual void activate();	// activate object
		virtual void deactivate();

		// functions to register dependent commands
		void addToActivationList ( Cmd * );
		void addToDeactivationList ( Cmd * );
		void addToUndoList ( Cmd * );
		void removeFromUndoList ( Cmd * );

		// register an Interface component used to execute this Cmd
		void registerInterface ( CmdInterface * );
		
		// access functions
		int active() { return ( _active ); }
		int hasUndo() { return ( _hasUndo ); }
		const char *const name() { return ( _name ); }
		virtual const char *const className() { return ( "Cmd" ); }
		virtual CmdType type() { return ( Action ); }
		
	protected:
		int _hasUndo;	// true if object supports undo
		static Cmd *_lastCmd;
		virtual void doCommand() = 0;	// Specific actions must be defined
		virtual void undoCommand() = 0;	// by derived classes
		Cmd ( const char *, int active = TRUE );
		virtual void revert();	// revert to object's previous state
		
	private:
		// Other commands which can be (de)activated when this command
		// is exec'd or undone
		CmdList *_activationList;
		CmdList *_deactivationList;
		CmdList *_undoList;

		int _active;			// is the current Cmd active?
		int _previouslyActive;	// previous _active value;
		char *_name;	// name of this Cmd
		List<CmdInterface *> _ci;
};
	
#endif
